'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.requestAnimationFrame = exports.cancelAnimationFrame = exports.shallowEqual = exports.deepEqual = exports.clamp = undefined;
exports.translateDOMPosition = translateDOMPosition;
exports.debounce = debounce;
exports.getJudgeFunction = getJudgeFunction;

var _translateDOMPositionXY = require('./dom/translateDOMPositionXY');

var _translateDOMPositionXY2 = _interopRequireDefault(_translateDOMPositionXY);

var _clamp2 = require('clamp');

var _clamp3 = _interopRequireDefault(_clamp2);

var _lodash = require('lodash.isequal');

var _lodash2 = _interopRequireDefault(_lodash);

var _shallowequal = require('shallowequal');

var _shallowequal2 = _interopRequireDefault(_shallowequal);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.clamp = _clamp3.default;
exports.deepEqual = _lodash2.default;
exports.shallowEqual = _shallowequal2.default;

var functionReturnTrue = function functionReturnTrue() {
  return true;
};
var functionReturnFalse = function functionReturnFalse() {
  return false;
};

var nativeRequestAnimationFrame = global.requestAnimationFrame || global.webkitRequestAnimationFrame || global.mozRequestAnimationFrame || global.oRequestAnimationFrame || global.msRequestAnimationFrame;

var cancelAnimationFrame = exports.cancelAnimationFrame = global.cancelAnimationFrame || global.webkitCancelAnimationFrame || global.mozCancelAnimationFrame || global.oCancelAnimationFrame || global.msCancelAnimationFrame || global.clearTimeout;

var animationFrameTime = 0;

var requestAnimationFrame = exports.requestAnimationFrame = nativeRequestAnimationFrame || function (callback) {
  var currTime = Date.now();
  var timeDelay = Math.max(0, 16 - (currTime - animationFrameTime));
  animationFrameTime = currTime + timeDelay;
  return global.setTimeout(function () {
    callback(Date.now());
  }, timeDelay);
};

function translateDOMPosition(style, x, y) {
  var initialRender = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

  if (initialRender) {
    style.left = x + 'px';
    style.top = y + 'px';
  } else {
    (0, _translateDOMPositionXY2.default)(style, x, y);
  }
}

function debounce(func, wait, context) {
  var setTimeoutFunc = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : setTimeout;
  var clearTimeoutFunc = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : clearTimeout;

  var timeout = void 0;

  var debouncer = function debouncer() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    debouncer.reset();

    var callback = function callback() {
      func.apply(context, args);
    };
    timeout = setTimeoutFunc(callback, wait);
  };

  debouncer.reset = function () {
    clearTimeoutFunc(timeout);
  };

  return debouncer;
}

function getJudgeFunction(fn) {
  if (typeof handleScrollX !== 'function') {
    return fn ? functionReturnTrue : functionReturnFalse;
  }
  return fn;
}